#!/usr/bin/bash
#/opt/skywire/scripts/skywire-autoconfig
#skywire autoconfiguration script for archlinux
#
#set NOAUTOCONFIG=true to avoid running the script in the postinstall
if [[ ${NOAUTOCONFIG} == true ]]; then
  #unset the env
  NOAUTOCONFIG=''
  echo "autoconfiguration disabled. to configure and start skywire run: skywire-autoconfig"
  exit 0
fi
#root portion of the configuration
if [[ $EUID -ne 0 ]]; then
   echo -e "Root permissions required" 1>&2
   exit 100
fi
#halt any running instance
[[ -f /usr/lib/systemd/system/skywire.service ]] && systemctl disable --now skywire.service 2> /dev/null
[[ -f /usr/lib/systemd/system/skywire-visor.service ]] && systemctl disable --now skywire-visor.service 2> /dev/null
[[ -f /usr/lib/systemd/system/skywire-autoconfig.service ]] && systemctl disable skywire-autoconfig.service 2> /dev/null
[[ -f /usr/lib/systemd/system/skywire-autoconfig-remote.service ]] && systemctl disable skywire-autoconfig-remote.service 2> /dev/null
[[ -f /usr/lib/systemd/system/skywire-hypervisor.service ]] && systemctl disable --now skywire-hypervisor.service 2> /dev/null
#make the logging of this script colorful
_nc='\033[0m'
_red='\033[0;31m'
_green='\033[0;32m'
_yellow='\033[0;33m'
_blue='\033[1;34m'
_purple='\033[0;35m'
_cyan='\033[0;36m'
_bold='\033[1m'
_1=${1}
_msg2() {
	(( QUIET )) && return
	local mesg=$1; shift
	printf "${_cyan} ->${_nc}${_bold} ${mesg}${_nc}\n" "$@"
}
_msg3() {
(( QUIET )) && return
local mesg=$1; shift
printf "${_blue} -->${_nc}${BOLD} ${mesg}${_nc}\n" "$@"
}
_errmsg1() {
	(( QUIET )) && return
	local mesg=$1; shift
	printf "${_red}>>> Error:${_nc}${_bold} ${mesg}${_nc}\n" "$@"
}
_warnmsg1() {
	(( QUIET )) && return
	local mesg=$1; shift
	printf "${_red}>>> Warning:${_nc}${_bold} ${mesg}${_nc}\n" "$@"
}
_errmsg2() {
	(( QUIET )) && return
	local mesg=$1; shift
	printf "${_red}>>> FATAL:${_bold} ${mesg}${_nc}\n" "$@"
}
_welcome(){
  _msg2 "register your public key:"
  _msg2 "${_blue}https://whitelist.skycoin.com/${_nc}"
  _msg2 "track uptime:"
  _msg2 "${_blue}http://ut.skywire.skycoin.com/uptimes${_nc}"
  _msg2 "support:"
  _msg2 "${_blue}https://t.me/skywire${_nc}"
}
_config_hypervisor() {
  #try to reuse existing configuration
  if [[ ! -f /opt/skywire/skywire.json ]]; then
    if [[ -f /etc/skywire-config.json ]]; then
      _warnmsg1 "Importing configuration from /etc/skywire-config.json"
      cp -b  /etc/skywire-config.json /opt/skywire/skywire.json
    fi
  fi
  ##generate hypervisor configuration##
  _msg3 "Generating skywire config with command:"
#  echo "  skywire-cli config gen -bipr --enable-auth ${_public_rpc}-o /opt/skywire/skywire.json"
  if [[ ! -z ${NOBIN} ]]; then
    echo "  go run ${NOBIN} config gen -bipr --enable-auth ${_public_rpc} ${_vpn_server}"
    go run ${NOBIN} config gen -bipr --enable-auth ${_public_rpc} ${_vpn_server} >> /dev/null 2>&1
    if [[ ${?} != 0 ]]; then
      #print the error!
      go run ${NOBIN} config gen -bipr --enable-auth ${_public_rpc} ${_vpn_server} >> /dev/null 2>&1
      _err=$?
      _errmsg2 "error generating skywire config"
      exit ${_err}
    fi
  else
    echo "  skywire-cli config gen -bipr --enable-auth ${_public_rpc} ${_vpn_server}"
    skywire-cli config gen -bipr --enable-auth ${_public_rpc} ${_vpn_server} >> /dev/null 2>&1
    if [[ ${?} != 0 ]]; then
      #print the error!
      skywire-cli config gen -bipr --enable-auth ${_public_rpc} ${_vpn_server}
      _err=$?
      _errmsg2 "error generating skywire config"
      exit ${_err}
    fi
  fi
  if [[ -f /opt/skywire/skywire.json ]]; then
    _msg2 "${_blue}Skywire${_nc} configuration updated!"
    echo "config path: /opt/skywire/skywire.json"
  else
    _errmsg2 "expected config file not found at /opt/skywire/skywire.json"
    exit 100
  fi
  echo "patching config with correct STUN servers.."
  sed -i 's/45.118.133.242/172.104.188.139/g' /opt/skywire/skywire.json
  sed -i 's/192.53.173.68/172.104.59.235/g' /opt/skywire/skywire.json
  sed -i 's/192.46.228.39/172.104.183.187/g' /opt/skywire/skywire.json
  sed -i 's/192.53.113.106/139.162.54.63/g' /opt/skywire/skywire.json
  sed -i 's/192.53.117.158/172.105.115.97/g' /opt/skywire/skywire.json
  sed -i 's/192.53.114.142/172.104.188.39/g' /opt/skywire/skywire.json
  sed -i 's/139.177.189.166/172.104.188.140/g' /opt/skywire/skywire.json
  sed -i 's/192.46.227.227/172.104.40.88/g' /opt/skywire/skywire.json
}
_config_visor() {
  if [[ ! -z ${NOBIN} ]]; then
    _pubkey=$(go run ${NOBIN} visor pk -i /opt/skywire/skywire-visor.json | tail -n1)
  else
    _pubkey=$(skywire-cli visor pk -i /opt/skywire/skywire.json | tail -n1)
  fi
  if [[ ${_1} == ${_pubkey} ]]; then
    _errmsg1 "The public key supplied is the current visor public key.
  Enter a different public key to set a remote hypervisor."
    exit 100
  fi
  _msg3 "using keys from existing config /opt/skywire/skywire.json"
  if [[ ! -f /opt/skywire/skywire-visor.json ]] ; then
    cp /opt/skywire/skywire.json /opt/skywire/skywire-visor.json
  fi
  _msg3 "Generating skywire-visor config with command:"
  if [[ ! -z ${NOBIN} ]]; then
    echo "go run ${NOBIN} config gen --hypervisor-pks ${_1} -bpr ${_vpn_server}"
    go run ${NOBIN} config gen --hypervisor-pks ${_1} -bpr ${_vpn_server} >> /dev/null 2>&1
    if [[ ${?} != 0 ]]; then
      #print the error!
      go run ${NOBIN} config gen --hypervisor-pks ${_1} -bpr ${_vpn_server} >> /dev/null 2>&1
      _err=$?
      _errmsg2 "error generating skywire config"
      exit ${_err}
    fi
  else
    echo "skywire-cli config gen --hypervisor-pks ${_1} -bpr ${_vpn_server}"
    skywire-cli config gen --hypervisor-pks ${_1} -bpr ${_vpn_server} >> /dev/null 2>&1
    if [[ ${?} != 0 ]]; then
      #print the error!
      skywire-cli config gen --hypervisor-pks ${_1} -bpr ${_vpn_server}
      _err=$?
      _errmsg2 "error generating skywire visor config"
      exit ${_err}
    fi
  fi
  _msg2 "visor configuration created"
  if [[ -f /opt/skywire/skywire-visor.json ]]; then
    echo "config path: /opt/skywire/skywire-visor.json"
  else
    _errmsg2 "expected config file not found at /opt/skywire/skywire-visor.json"
    exit 100
  fi
  echo "patching config with correct STUN servers.."
  sed -i 's/45.118.133.242/172.104.188.139/g' /opt/skywire/skywire-visor.json
  sed -i 's/192.53.173.68/172.104.59.235/g' /opt/skywire/skywire-visor.json
  sed -i 's/192.46.228.39/172.104.183.187/g' /opt/skywire/skywire-visor.json
  sed -i 's/192.53.113.106/139.162.54.63/g' /opt/skywire/skywire-visor.json
  sed -i 's/192.53.117.158/172.105.115.97/g' /opt/skywire/skywire-visor.json
  sed -i 's/192.53.114.142/172.104.188.39/g' /opt/skywire/skywire-visor.json
  sed -i 's/139.177.189.166/172.104.188.140/g' /opt/skywire/skywire-visor.json
  sed -i 's/192.46.227.227/172.104.40.88/g' /opt/skywire/skywire-visor.json
  if [[ -z ${NOBIN} ]]; then
    _msg3 "Enabling skywire-visor systemd service:
    systemctl enable --now skywire-visor.service"
    systemctl enable --now skywire-visor.service 2> /dev/null
    _msg2 "${_blue}Skywire${_nc} starting in visor-only mode"
    _pubkey=$(skywire-cli visor pk -i /opt/skywire/skywire-visor.json | tail -n1)
  else
    _msg2 "to start ${_blue}Skywire${_nc}:"
    echo "go run ${NOBIN/skywire-cli\/skywire-cli.go/skywire-visor\/skywire-visor.go}"
    _pubkey=$(go run ${NOBIN} visor pk -i /opt/skywire/skywire-visor.json | tail -n1)
  fi
  _msg2 "Visor Public Key: ${_green}${_pubkey}${_nc}"
  _welcome
}
_config_tips() {
  _msg2 "execute the following command on OTHER NODES to set this one as their hypervisor:"
  echo -e "${_cyan}skywire-autoconfig ${_yellow}${_pubkey}${_nc}"
  if [[ ! -z ${_public_rpc} ]]; then
  _msg2 "or by ip address:"
  echo -e "${_cyan}skywire-autoconfig ${_yellow}\$(skywire-cli --rpc ${_lanip}:3435 visor pk)${_nc}"
  fi
  _msg2 "to see this text again run: ${_cyan}skywire-autoconfig${_nc}"
}
_msg2 "Configuring skywire"
#only use public rpc flag if this is the hypervisor
if [[ $(hostname) == "hypervisor" ]]; then
  _public_rpc="--public-rpc "
fi
#enable VPN server automatically
if [[ ${VPNSERVER} == true ]]; then
  _vpn_server="--vpn-server-enable "
fi
#use NOBIN to specify the cmd/slywire-cli/skywire-cli.go file in the source code
if [[ ! -z ${NOBIN} ]]; then
  if [[ ! -f ${NOBIN} ]]; then
    unset NOBIN
  fi
  if [[ ${NOBIN} != *".go" ]]; then
    unset NOBIN
  fi
fi
#update the hypervisor config file or create it
_config_hypervisor
#script takes public key as argument to configure remote hypervisor
if [[ -z $1 ]] ; then #no pk as argument
  #if no pubkey as argument the configs that exist are updated
  if [[ ! -f /opt/skywire/skywire-visor.json ]]; then #no visor config
    ####>>start hypervisor if visor config does not exist<<####
    if [[ -z ${NOBIN} ]]; then
      _msg3 "Enabling skywire systemd service..
    systemctl enable --now skywire.service"
      systemctl enable --now skywire.service 2> /dev/null
      _pubkey=$(skywire-cli visor pk -i /opt/skywire/skywire.json | tail -n1)
      #_vpnurl=$(skywire-cli visor vpn url)
    else
      _msg2 "to start ${_blue}Skywire${_nc}:"
      export _skywirevisor="${NOBIN/skywire-cli\/skywire-cli.go/skywire-visor\/skywire-visor.go}"
      echo -e "${_cyan}sudo go run ${_skywirevisor} -c /opt/skywire/skywire.json${_nc}"
      if [[ ${NOBIN1} == true ]]; then
        sudo go run ${_skywirevisor} -c /opt/skywire/skywire.json
      fi
        _pubkey=$(go run ${NOBIN} visor pk -i /opt/skywire/skywire.json | tail -n1)
    fi
      _msg2 "Visor Public Key:
${_green}${_pubkey}${_nc}"
      _msg2 "Starting now on:
    ${_red}http://127.0.0.1:8000${_nc}"
      _msg2 "Use the vpn:
${_red}http://127.0.0.1:8000/#/vpn/${_pubkey}/${_nc}"
      _lanip="$(ip addr show | grep -w inet | grep -v 127.0.0.1 | awk '{ print $2}' | cut -d "/" -f 1)"
      _msg2 "Access hypervisor UI from local network here:
    ${_yellow}http://${_lanip}:8000${_nc}"
      _welcome
      _config_tips
  else
    #The visor config exists
    _msg3 "updating skywire-visor config with command:"
    echo "skywire-cli config gen -bprx ${_vpn_server}"
    if [[ ! -z ${NOBIN} ]]; then
      go run ${NOBIN} config gen --hypervisor-pks ${_1} -bpr ${_vpn_server} >> /dev/null 2>&1
      if [[ ${?} != 0 ]]; then
        #print the error!
        go run ${NOBIN} config gen --hypervisor-pks ${_1} -bpr ${_vpn_server} >> /dev/null 2>&1
        _err=$?
        _errmsg2 "error generating skywire config"
        exit ${_err}
      fi
    else
      skywire-cli config gen -bprx${_vpn_server} >> /dev/null 2>&1
      if [[ ${?} != 0 ]]; then
        #print the error!
        skywire-cli config gen -bprx
        _err=$?
        _errmsg2 "error generating skywire visor config"
        exit ${_err}
      fi
      _msg2 "${_blue}Skywire${_nc} visor configuration updated!"
      if [[ -f /opt/skywire/skywire-visor.json ]]; then
        echo "config path: /opt/skywire/skywire-visor.json"
      else
        _errmsg2 "expected config file not found at /opt/skywire/skywire-visor.json"
        exit 100
      fi
    fi
    if [[ -z ${NOBIN} ]]; then
      echo "patching config with correct STUN servers.."
      sed -i 's/45.118.133.242/172.104.188.139/g' /opt/skywire/skywire-visor.json
      sed -i 's/192.53.173.68/172.104.59.235/g' /opt/skywire/skywire-visor.json
      sed -i 's/192.46.228.39/172.104.183.187/g' /opt/skywire/skywire-visor.json
      sed -i 's/192.53.113.106/139.162.54.63/g' /opt/skywire/skywire-visor.json
      sed -i 's/192.53.117.158/172.105.115.97/g' /opt/skywire/skywire-visor.json
      sed -i 's/192.53.114.142/172.104.188.39/g' /opt/skywire/skywire-visor.json
      sed -i 's/139.177.189.166/172.104.188.140/g' /opt/skywire/skywire-visor.json
      sed -i 's/192.46.227.227/172.104.40.88/g' /opt/skywire/skywire-visor.json
      _msg3 "Enabling skywire-visor systemd service:
    systemctl enable --now skywire-visor.service"
    systemctl enable --now skywire-visor.service 2> /dev/null
      _msg2 "${_blue}Skywire${_nc} starting in visor-only mode"
    else
      _msg2 "to start ${_blue}Skywire${_nc}:"
      echo "sudo go run ${NOBIN/skywire-cli\/skywire-cli.go/skywire-visor\/skywire-visor.go} -c /opt/skywire/skywire-visor.json"
    fi
    _pubkey=$(skywire-cli visor pk -i /opt/skywire/skywire-visor.json | tail -n1)
    _msg2 "Visor Public Key: ${_green}${_pubkey}${_nc}"
    _welcome
  fi
else #[[ ! -z $1 ]]
  _config_visor
fi
